# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

makesky_rcpp <- function(albedo = 0.5, turbidity = 3.0, elevation = 10.0, azimuth_deg = 90, resolution = 2048L, number_cores = 1L, model = "hosek", prg_dataset = "", altitude = 0.0, visibility = 50.0, render_mode = "all", lambda_nm = NULL, below_horizon = TRUE) {
    .Call(`_skymodelr_makesky_rcpp`, albedo, turbidity, elevation, azimuth_deg, resolution, number_cores, model, prg_dataset, altitude, visibility, render_mode, lambda_nm, below_horizon)
}

calculate_sun_brightness_rcpp <- function(albedo = 0.5, turbidity = 3.0, elevation = 10.0, azimuth_deg = 90.0, model = "hosek", prg_dataset = "", altitude = 0.0, visibility = 50.0, lambda_nm = NULL) {
    .Call(`_skymodelr_calculate_sun_brightness_rcpp`, albedo, turbidity, elevation, azimuth_deg, model, prg_dataset, altitude, visibility, lambda_nm)
}

calculate_sun_radiance_band_rcpp <- function(albedo = 0.5, turbidity = 3.0, elevation = 10.0, azimuth_deg = 90.0, model = "hosek", prg_dataset = "", altitude = 0.0, visibility = 50.0, lambda_nm = NULL) {
    .Call(`_skymodelr_calculate_sun_radiance_band_rcpp`, albedo, turbidity, elevation, azimuth_deg, model, prg_dataset, altitude, visibility, lambda_nm)
}

calculate_raw_prague_radiance <- function(phi, theta, lambda_nm, elevation, albedo, altitude, visibility, azimuth, number_cores = 1L, prg_dataset = "", render_mode = "all") {
    .Call(`_skymodelr_calculate_raw_prague_radiance`, phi, theta, lambda_nm, elevation, albedo, altitude, visibility, azimuth, number_cores, prg_dataset, render_mode)
}

calculate_raw_prague <- function(phi, theta, elevation, albedo, altitude, visibility, azimuth, number_cores = 1L, prg_dataset = "", render_mode = "all") {
    .Call(`_skymodelr_calculate_raw_prague`, phi, theta, elevation, albedo, altitude, visibility, azimuth, number_cores, prg_dataset, render_mode)
}

cie_1931_2deg_rcpp <- function() {
    .Call(`_skymodelr_cie_1931_2deg_rcpp`)
}

spd_values_rcpp <- function(lambda_nm, spd_type = "D65") {
    .Call(`_skymodelr_spd_values_rcpp`, lambda_nm, spd_type)
}

make_starfield_rcpp <- function(stars, resolution = 2048L, zero_point = 1.0, lon_deg = 0.0, lat_deg = 0.0, jd = 2451545.0, turbidity = 3.0, ozone_du = 300.0, altitude = 0.0, star_width = 1.0, use_rgb = TRUE, atmosphere_effects = TRUE, upper_hemisphere_only = TRUE, number_cores = 1L) {
    .Call(`_skymodelr_make_starfield_rcpp`, stars, resolution, zero_point, lon_deg, lat_deg, jd, turbidity, ozone_du, altitude, star_width, use_rgb, atmosphere_effects, upper_hemisphere_only, number_cores)
}

