% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/radiometry.R
\name{compute_K_eff}
\alias{compute_K_eff}
\title{Compute effective luminous efficacy for a chosen SPD}
\usage{
compute_K_eff(
  spd_type = c("D65", "BB5778"),
  wavelength_grid = NULL,
  V_lambda = NULL
)
}
\arguments{
\item{spd_type}{SPD shape. Either \code{"D65"} or \code{"BB5778"}.}

\item{wavelength_grid}{Optional numeric vector of wavelengths (nm).}

\item{V_lambda}{Optional numeric vector of photopic V(lambda) samples.}
}
\value{
Effective luminous efficacy in lm/W.
}
\description{
Compute effective luminous efficacy for a chosen SPD
}
\keyword{internal}
