% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_moon_image_latlong.R
\name{normalize}
\alias{normalize}
\title{Normalize a vector to unit length}
\usage{
normalize(v)
}
\arguments{
\item{v}{Numeric vector to normalise.}
}
\description{
Scale a numeric vector so it has unit Euclidean length.
}
\keyword{internal}
