% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_sky_data.R
\name{download_sky_data}
\alias{download_sky_data}
\title{Download Prague Sky Model Coefficient Data}
\usage{
download_sky_data(sea_level = TRUE, wide_spectrum = FALSE)
}
\arguments{
\item{sea_level}{Default \code{TRUE}. Download the sea-level-only data.
Set to \code{FALSE} to download the full-altitude dataset.}

\item{wide_spectrum}{Default \code{FALSE}. If \code{TRUE}, downloads the
wide-spectrum (55-channel, polarised) version. Valid only when \code{sea_level = TRUE}.}
}
\value{
Invisibly, the full path to the data file.
}
\description{
This model which allows for sun angles below the horizon,
wide spectral ranges including infrared, polarization, as well
rendering at altitude) need to download a coefficient file. There are three versions
of this dataset, listed here in order of increasing size:
}
\details{
\tabular{lll}{
   Argument combination \tab File \tab Size \cr
   \verb{sea_level = TRUE,  wide_spectrum = FALSE} \tab \code{SkyModelDatasetGround.dat} \tab 107MB \cr
   \verb{sea_level = TRUE,  wide_spectrum = TRUE} \tab \code{PragueSkyModelDatasetGroundInfra.dat} \tab 574MB \cr
   \code{sea_level = FALSE} \tab \code{SkyModelDataset.dat} \tab 2.4GB \cr
}
}
\examples{
\dontshow{if (interactive() || identical(Sys.getenv("IN_PKGDOWN"), "true")) withAutoprint(\{ # examplesIf}
# Standard (11-channel, sea-level) coefficients
download_sky_data()

# Wide-spectrum sea-level coefficients
download_sky_data(wide_spectrum = TRUE)

# Full altitude-range coefficients
download_sky_data(sea_level = FALSE)
\dontshow{\}) # examplesIf}
}
