% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sky_data_cache.R
\name{clear_sky_data}
\alias{clear_sky_data}
\title{Clear cached sky data files}
\usage{
clear_sky_data(files = NULL, ask = interactive())
}
\arguments{
\item{files}{Default \code{NULL}. Character vector of cached file basenames to remove.}

\item{ask}{Default \code{interactive()}. Whether to ask for confirmation before deleting files.}
}
\value{
Invisibly, the paths successfully removed.
}
\description{
Removes files cached by \code{\link[=download_sky_data]{download_sky_data()}} under
\code{tools::R_user_dir("skymodelr", "data")}.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
clear_sky_data()
\dontshow{\}) # examplesIf}
}
